package org.kohsuke.rng2srng;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.xml.transform.stream.StreamResult;

import org.kohsuke.rng2srng.datatype.DatatypeLibraryFactoryImpl;
import org.relaxng.datatype.helpers.DatatypeLibraryLoader;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.thaiopensource.relaxng.IncorrectSchemaException;
import com.thaiopensource.relaxng.impl.Pattern;
import com.thaiopensource.relaxng.impl.SchemaBuilderImpl;
import com.thaiopensource.relaxng.impl.SchemaPatternBuilder;
import com.thaiopensource.relaxng.parse.Parseable;
import com.thaiopensource.relaxng.parse.sax.SAXParseable;
import com.thaiopensource.relaxng.util.ErrorHandlerImpl;
import com.thaiopensource.relaxng.util.Jaxp11XMLReaderCreator;

/**
 * 
 * 
 * @author Kohsuke Kawaguchi (kk@kohsuke.org)
 */
public class Driver {
    private static void usage( String msg ) {
        if(msg!=null)   System.err.println(msg);
        
        System.err.println(
            "Usage: rng2srng [-c] <grammar file>\n"+
            "  translates RELAX NG into the simplified form\n"+
            "\n"+
            "  -c : use the compact syntax as the input");
        System.exit(-1);
    }

    public static void main( String[] args ) throws Exception {
        
        URL grammar = null;
        boolean compact = false;
        
        for( int i=0; i<args.length; i++ ) {
            String arg = args[i];
            
            if( arg.equals("-c") ) {
                compact=true;
                continue;
            }
            
            if( arg.charAt(0)!='-') {
                if( grammar!=null )
                    usage("more than one grammars are specfieid");
                grammar = new File(arg).toURL();
                continue;
            }
            
            usage("unknown option: "+arg);
        }
        
        if(grammar==null)
            usage(null);
        
        if(compact) {
            Translator.translateCompactToResult(
                new InputSource(grammar.toExternalForm()),
                new ErrorHandlerImpl(),
                new StreamResult(System.out) );
        } else {
            Translator.translateXmlToResult(
                new InputSource(grammar.toExternalForm()),
                new ErrorHandlerImpl(),
                new StreamResult(System.out) );
        }
    }
}
